==> elmo-1.3.3.tgz: package/src/lib/router.tsx <==
import { parse } from '../client/url'
import { h, render } from 'preact'

const browser = typeof window !== 'undefined'
const regexps: { [s: string]: RegExp } = {}
const location = browser && window.location
const history = browser && window.history

export type Route = {
  path: string
  regexp: string
  params: (string | number)[]
}

export type URL = {
  protocol: string
  host: string
  port: number
  hostname: string
  hash: string
  search: string
  query: {
    [s: string]: string | boolean
  }
  pathname: string
  href: string
}

type PartialRoute = {
  path: string
  params: {
    [s: string]: string
  }
}

interface W extends Window {
  _ELMO_ROUTES_: Route[]
}

/**
 * Router
 */

export class Router {
  routes: Route[]
  cache: { [path: string]: any }

  constructor() {
    this.onPopState = this.onPopState.bind(this)
    this.routes = []
    this.cache = {}

    if (browser) {
      this.routes = (window as W)._ELMO_ROUTES_ || []
      window.addEventListener('popstate', this.onPopState)
    }
  }

  async onPopState(_e: PopStateEventInit) {
    await this.replace(this.getURL())
  }

  async push(url: string) {
    const err = await this.possLoad(url)
    if (err) return this.error(err)
    if (history) window.history.pushState(null, document.title, url)
    else if (location) location.href = url
  }

  error(err: Error) {
    err.message = '[elmo/router] ' + err.message
    throw err
  }

  async replace(url: string) {
    const err = await this.possLoad(url)
    if (err) return this.error(err)
    if (history) window.history.replaceState(null, document.title, url)
    else if (location) location.href = url
  }

  async possLoad(url: string) {
    try {
      await this.load(url)
      return null
    } catch (e) {
      return e
    }
  }

  async load(as: string) {
    const url = parse(as)
    const route = this.findRoute(url.pathname)
    if (!route) throw new Error(`no route found: ${as}`)

    const Component = this.cache[route.path] || (await this.get(route.path))
    await this.renderComponent(Component, url, route.params)
    this.cache[route.path] = Component
  }

  async get(url: string) {
    let component = null

    try {
      const res = await window.fetch(url)
      const code = await res.text()
      component = this.evaluate(code)
    } catch (e) {
      console.error(e)
    }

    return component
  }

  async update(Component: any, path: string) {
    const url = parse(document.location.href)
    const route = this.findRoute(url.pathname)

    // TODO just return here? probably not
    if (!route) return

    // cache this component
    this.cache[path] = Component

    // render the component if we're on that page
    if (route.path === path) {
      await this.renderComponent(Component, url, route.params || {})
    }
  }

  async renderComponent(Component: any, url: URL, params: Object) {
    const root = document.getElementById('elmo.page')
    let props = {}

    if (Component.getInitialProps) {
      const before = window.location.href
      props = await Component.getInitialProps({
        url,
        params
      })

      // sometimes getinitialprops will redirect,
      // in that case, we'll want to ignore this
      // render since it will be old.
      if (before !== window.location.href) {
        return
      }
    }

    if (root instanceof HTMLElement && root.firstChild instanceof HTMLElement) {
      render(h(Component, props || {}), root, root.firstChild)
    }
  }

  getURL() {
    if (location) {
      return `${location.pathname || ''}${location.search || ''}`
    } else {
      return ''
    }
  }

  evaluate(script: string) {
    const exports = { default: undefined }
    const module = { exports: exports }

    eval(script) // eslint-disable-line no-eval

    if (module.exports && module.exports.default) {
      return module.exports.default
    } else {
      return module.exports
    }
  }

  private findRoute(path: string): PartialRoute | void {
    const routes = this.routes
    for (let i = 0, len = routes.length; i < len; i++) {
      const route = routes[i]
      const regexp = (regexps[route.regexp] =
        regexps[route.regexp] || new RegExp(route.regexp))
      const m = regexp.exec(path)
      if (!m) continue

      const params: { [key: string]: string } = {}
      for (let i = 1, len = m.length; i < len; ++i) {
        var key = route.params[i - 1]
        if (key) params[key] = decodeURIComponent(m[i])
      }

      // trim off any '/' from front or back
      let routePath = route.path.replace(/^\/+|\/+$/g, '')
      routePath = routePath === '' ? routePath : '/' + routePath

      return {
        path: routePath + '/index.js',
        params: params
      }
    }
  }
}

export default new Router()

==> elmo-2.0.1.tgz: package/src/lib/router.tsx <==
import { h, Component, render as renderDOM } from 'preact'
import { parse } from '../client/url'

const browser = typeof window !== 'undefined'
const regexps: { [s: string]: RegExp } = {}
const location = browser && window.location
const history = browser && window.history

export type URL = {
  protocol: string
  host: string
  port: number
  hostname: string
  hash: string
  search: string
  query: {
    [s: string]: string | boolean
  }
  pathname: string
  href: string
}

type PartialRoute = {
  path: string
  params: {
    [s: string]: string
  }
}

export type Route = {
  path: string
  regexp: string
  params: (string | number)[]
}

/**
 * Router
 */

export default class Router {
  routes: Route[]
  cache: { [path: string]: any }

  constructor(routes: Route[] = []) {
    this.routes = routes
    this.cache = {}

    if (browser) {
      this.onPopState = this.onPopState.bind(this)
      window.addEventListener('popstate', this.onPopState)
    }
  }

  async onPopState(_e: PopStateEventInit) {
    await this.replace(this.getURL())
  }

  async push(url: string) {
    const err = await this.possLoad(url)
    if (err) return this.error(err)
    if (history) window.history.pushState(null, document.title, url)
    else if (location) location.href = url
  }

  error(err: Error) {
    err.message = '[elmo/router] ' + err.message
    throw err
  }

  async replace(url: string) {
    const err = await this.possLoad(url)
    if (err) return this.error(err)
    if (history) window.history.replaceState(null, document.title, url)
    else if (location) location.href = url
  }

  async possLoad(url: string) {
    try {
      await this.load(url)
      return null
    } catch (e) {
      return e
    }
  }

  async load(as: string) {
    const url = parse(as)
    const route = this.findRoute(url.pathname)
    if (!route) throw new Error(`no route found: ${as}`)

    const Component = this.cache[route.path] || (await this.get(route.path))
    await this.renderComponent(Component, url, route.params)
    this.cache[route.path] = Component
  }

  async get(url: string) {
    let component = null

    try {
      const res = await window.fetch(url)
      const code = await res.text()
      component = this.evaluate(code)
    } catch (e) {
      console.error(e)
    }

    return component
  }

  async update(Component: any, path: string) {
    const url = parse(document.location.href)
    const route = this.findRoute(url.pathname)

    // TODO just return here? probably not
    if (!route) {
      document.location.pathname = '/404'
      return
    }

    // cache this component
    this.cache[path] = Component

    // render the component if we're on that page
    if (route.path === path) {
      await this.renderComponent(Component, url, route.params || {})
    }
  }

  async renderComponent(Component: any, url: URL, params: Object) {
    const root = document.getElementById('elmo.page')
    let firstChild = root ? root.firstChild : undefined
    if (!(root instanceof HTMLElement)) return

    const render = (props = {}) => {
      firstChild = renderDOM(
        h(Provider, { router: this }, h(Component, props)),
        root,
        firstChild as Element
      )
    }

    const initial = { render, params, url }
    if (Component) {
      if (!Component.getInitialProps) {
        return render(initial)
      }

      Component.getInitialProps(initial)
        .then((props: any) => {
          props && render(props)
        })
        .catch((err: Error) => {
          // TODO: error page
          console.error(err)
        })
    }
  }

  getURL() {
    if (location) {
      return `${location.pathname || ''}${location.search || ''}`
    } else {
      return ''
    }
  }

  evaluate(script: string) {
    const exports = { default: undefined }
    const module = { exports: exports }

    eval(script) // eslint-disable-line no-eval

    if (module.exports && module.exports.default) {
      return module.exports.default
    } else {
      return module.exports
    }
  }

  private findRoute(path: string): PartialRoute | void {
    const routes = this.routes
    for (let i = 0, len = routes.length; i < len; i++) {
      const route = routes[i]
      const regexp = (regexps[route.regexp] =
        regexps[route.regexp] || new RegExp(route.regexp))
      const m = regexp.exec(path)
      if (!m) continue

      const params: { [key: string]: string } = {}
      for (let i = 1, len = m.length; i < len; ++i) {
        var key = route.params[i - 1]
        if (key) params[key] = decodeURIComponent(m[i])
      }

      // trim off any '/' from front or back
      let routePath = route.path.replace(/^\/+|\/+$/g, '')
      routePath = routePath === '' ? routePath : '/' + routePath

      return {
        path: routePath + '/index.js',
        params: params
      }
    }
  }
}

namespace Provider {
  export interface Props {
    router: Router
  }
}

class Provider extends Component<Provider.Props, never> {
  constructor(props: Provider.Props) {
    super(props)
  }

  getChildContext() {
    return {
      router: this.props.router
    }
  }

  render(): JSX.Element | null {
    return this.props.children ? this.props.children[0] : null
  }
}
